window.Gleam.initSearch([{"doc":"persevero","title":"persevero","content":"# persevero\n\n[![Package Version](https://img.shields.io/hexpm/v/persevero)](https://hex.pm/packages/persevero)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/persevero/)\n\nExecute fallible operations multiple times.\n\n## Installation\n\n```sh\ngleam add persevero\n```\n\n## Usage\n\nA simple example:\n\n```gleam\nimport gleam/http/request\nimport gleam/httpc\nimport gleam/io\nimport persevero\n\npub fn main() {\n  let assert Ok(request) = request.to(&quot;https://www.apple.com&quot;)\n\n  let response = {\n    use &lt;- persevero.execute(\n      wait_stream: persevero.exponential_backoff(50, 2),\n      allow: persevero.all_errors,\n      mode: persevero.MaxAttempts(3),\n    )\n\n    httpc.send(request)\n  }\n\n  case response {\n    Ok(response) if response.status == 200 -&gt; io.debug(&quot;Give me #prawducks. 😃&quot;)\n    _ -&gt; io.debug(&quot;Guess I&#39;ll dev on Linux. 😔&quot;)\n  }\n}\n```\n\nA ridiculous example:\n\n```gleam\nimport gleam/http/request\nimport gleam/httpc\nimport gleam/io\nimport persevero\n\npub fn main() {\n  let assert Ok(request) = request.to(&quot;https://www.apple.com&quot;)\n\n  let response =\n    persevero.custom_backoff(wait_time: 1000, next_wait_time: fn(previous) {\n      { previous + 100 } * 2\n    })\n    |&gt; persevero.apply_multiplier(3)\n    |&gt; persevero.apply_jitter(20)\n    |&gt; persevero.apply_cap(10_000)\n    |&gt; persevero.apply_constant(7)\n    |&gt; persevero.execute(\n      allow: fn(error) {\n        case error {\n          httpc.InvalidUtf8Response -&gt; True\n          _ -&gt; False\n        }\n      },\n      mode: persevero.MaxAttempts(10),\n      operation: fn() { httpc.send(request) },\n    )\n\n  case response {\n    Ok(response) if response.status == 200 -&gt; io.debug(&quot;Give me #prawducks. 😃&quot;)\n    _ -&gt; io.debug(&quot;Guess I&#39;ll dev on Linux. 😔&quot;)\n  }\n}\n```\n\nInstead of defining a maximum number of attempts, you can use an expiry mode.\nThis will cause the operation to cease retrying after a specified duration.\n\n```gleam\nuse &lt;- persevero.execute(\n  wait_stream: persevero.constant_backoff(100),\n  allow: persevero.all_errors,\n  mode: persevero.Expiry(10_000),\n)\n```\n\nUse raw [`yielder`](https://hexdocs.pm/gleam_yielder/gleam/yielder.html)s for\nultimate wait stream manipulation:\n\n```gleam\nuse &lt;- persevero.execute(\n  wait_stream: yielder.repeat(5)\n    |&gt; yielder.intersperse(10)\n    |&gt; yielder.cycle,\n  allow: persevero.all_errors,\n  mode: persevero.MaxAttempts(100),\n)\n```\n\nNote that `persevero` generates a final wait stream with an initial 0 value, in\nthe `execute` function, after all transformations to the stream have been\napplied, so the first attempt will never sleep. You do not need to account for\nthis in your custom wait stream.\n\n## Targets\n\n`persevero` supports the Erlang target.\n\n## Inspiration\n\nThe feature set, and some of the design, is inspired by the\n[`retry`](https://hex.pm/packages/retry) library for Elixir.\n","url":"index.html"},{"doc":"persevero","title":"Error","content":"pub type Error(a) {\n  RetriesExhausted(errors: List(a))\n  TimeExhausted(errors: List(a))\n  UnallowedError(error: a)\n}\n Represents errors that can occur during execution attempts.\nRetriesExhausted(errors: List(a))\n Indicates that all execution attempts have been exhausted. Contains an\n ordered list of all errors encountered during the execution attempts.\n\nTimeExhausted(errors: List(a))\n Indicates that the maximum duration for execution has been reached.\n Contains an ordered list of all errors encountered during the execution\n\nUnallowedError(error: a)\n Indicates that an error that wasn&#39;t allowed was encountered. Contains the\n specific error that caused execution to stop.\n\nSynonyms:\npersevero.Error\npersevero Error","url":"persevero.html#Error"},{"doc":"persevero","title":"Mode","content":"pub type Mode {\n  MaxAttempts(Int)\n  Expiry(Int)\n}\n Configures the retry mode.\nMaxAttempts(Int)\n Specifies the maximum number of attempts to make.\n\nExpiry(Int)\n Specifies the maximum duration, in ms, to make attempts for.\n\n Note that `Expiry` mode does not prevent the current wait time from\n spilling over past the timeout. For example, if you&#39;re currently under the\n expiry timeout by 1s and your current wait time is 10s, this wait time,\n and the attempt on the operation, will still be run, resulting in\n spillover. Also note that the duration measured includes the time it takes\n to run your operation.\n\nSynonyms:\npersevero.Mode\npersevero Mode","url":"persevero.html#Mode"},{"doc":"persevero","title":"all_errors","content":"pub fn all_errors(: a) -&gt; Bool\n Convenience function that you can supply to `execute`&#39;s `allow` parameter to\n allow all errors.\nSynonyms:\npersevero.all_errors\npersevero all_errors","url":"persevero.html#all_errors"},{"doc":"persevero","title":"apply_cap","content":"pub fn apply_cap(\n  wait_stream wait_stream: Yielder(Int),\n  max_wait_time max_wait_time: Int,\n) -&gt; Yielder(Int)\n Caps each wait time at a maximum value.\nSynonyms:\npersevero.apply_cap\npersevero apply_cap","url":"persevero.html#apply_cap"},{"doc":"persevero","title":"apply_constant","content":"pub fn apply_constant(\n  wait_stream wait_stream: Yielder(Int),\n  adjustment adjustment: Int,\n) -&gt; Yielder(Int)\n Adds a constant integer to each wait time.\nSynonyms:\npersevero.apply_constant\npersevero apply_constant","url":"persevero.html#apply_constant"},{"doc":"persevero","title":"apply_jitter","content":"pub fn apply_jitter(\n  wait_stream wait_stream: Yielder(Int),\n  upper_bound upper_bound: Int,\n) -&gt; Yielder(Int)\n Adds a random integer between [1, `upper_bound`] to each wait time.\nSynonyms:\npersevero.apply_jitter\npersevero apply_jitter","url":"persevero.html#apply_jitter"},{"doc":"persevero","title":"apply_multiplier","content":"pub fn apply_multiplier(\n  wait_stream wait_stream: Yielder(Int),\n  factor factor: Int,\n) -&gt; Yielder(Int)\n Multiplies each wait time by a constant factor.\nSynonyms:\npersevero.apply_multiplier\npersevero apply_multiplier","url":"persevero.html#apply_multiplier"},{"doc":"persevero","title":"constant_backoff","content":"pub fn constant_backoff(wait_time wait_time: Int) -&gt; Yielder(Int)\n Produces a ms wait stream with a constant wait time.\n Ex: 500ms, 500ms, 500ms, ...\nSynonyms:\npersevero.constant_backoff\npersevero constant_backoff","url":"persevero.html#constant_backoff"},{"doc":"persevero","title":"custom_backoff","content":"pub fn custom_backoff(\n  wait_time wait_time: Int,\n  next_wait_time next_wait_time: fn(Int) -&gt; Int,\n) -&gt; Yielder(Int)\n Produces a custom ms wait stream.\nSynonyms:\npersevero.custom_backoff\npersevero custom_backoff","url":"persevero.html#custom_backoff"},{"doc":"persevero","title":"execute","content":"pub fn execute(\n  wait_stream wait_stream: Yielder(Int),\n  allow allow: fn(a) -&gt; Bool,\n  mode mode: Mode,\n  operation operation: fn() -&gt; Result(b, a),\n) -&gt; Result(b, Error(a))\n Initiates the execution process with the specified operation.\n\n `allow` sets the logic for determining whether an error should trigger\n another attempt. Expects a function that takes an error and returns a\n boolean. Use this function to match on the encountered error and return\n `True` for errors that should trigger another attempt, and `False` for\n errors that should not. To allow all errors, use `all_errors`.\nSynonyms:\npersevero.execute\npersevero execute","url":"persevero.html#execute"},{"doc":"persevero","title":"exponential_backoff","content":"pub fn exponential_backoff(\n  wait_time wait_time: Int,\n  factor factor: Int,\n) -&gt; Yielder(Int)\n Produces a ms wait stream that increases exponentially for each attempt.\n time:\n Ex: 500ms, 1000ms, 2000ms, ...\nSynonyms:\npersevero.exponential_backoff\npersevero exponential_backoff","url":"persevero.html#exponential_backoff"},{"doc":"persevero","title":"linear_backoff","content":"pub fn linear_backoff(\n  wait_time wait_time: Int,\n  step step: Int,\n) -&gt; Yielder(Int)\n Produces a ms wait stream that increases linearly for each attempt.\n Ex: 500ms, 1000ms, 1500ms, ...\nSynonyms:\npersevero.linear_backoff\npersevero linear_backoff","url":"persevero.html#linear_backoff"},{"doc":"persevero","title":"no_backoff","content":"pub fn no_backoff() -&gt; Yielder(Int)\n Produces a 0ms wait stream.\n Ex: 0ms, 0ms, 0ms, ...\nSynonyms:\npersevero.no_backoff\npersevero no_backoff","url":"persevero.html#no_backoff"},{"doc":"persevero","title":"persevero","content":" `persevero` executes a fallible operation multiple times.\n\n ```gleam\n import gleam/http/request\n import gleam/httpc\n import gleam/io\n import persevero\n\n pub fn main() {\n   let assert Ok(request) = request.to(&quot;https://www.apple.com&quot;)\n\n   let response = {\n     use &lt;- persevero.execute(\n       wait_stream: persevero.exponential_backoff(50, 2),\n       allow: persevero.all_errors,\n       mode: persevero.MaxAttempts(3),\n     )\n\n     httpc.send(request)\n   }\n\n   case response {\n     Ok(response) if response.status == 200 -&gt; io.debug(&quot;Give me #prawducks. 😃&quot;)\n     _ -&gt; io.debug(&quot;Guess I&#39;ll dev on Linux. 😔&quot;)\n   }\n }\n ```","url":"persevero.html"}]);